%% RNA counts and transcription site analysis based on FISH-Quant detection
% Script to calculate transcription site properties and RNA counts after FISH-quant analysis (Mueller et al., 2013). For more information on detection see FISH-quant documentation.
% To analyze data, use FISH-quant batch mode and save detection results as matlab array. 
% Note that FISH-quant raw data contains more information not included in
% this worfkflow. Script is designed for single cell analysis with
% segmented nuclei, cytoplasm and active TSS

% These parameters need to be defined gene specifically (MDM2 is used as an example here)
    gene = 'MDM2';  % target 
    l = 31372;      % length (bp)    
    v = 180000;     % average RNAP2 elongation in nt/h, here 50 nt/sec = 180000 nt/h 
    e = 0.49;       % correction factor for probe position from Trans-Quant (Bahar-Halpern and Itzkovitz, 2016)         
    k = 1.5;        % correction factor for RNAPol2 occupancy 
    n = 3;          % number of genomic loci (validate by DNA FISH)
  
%% Import and rearrange FISH-quant data
% Write DatasetIdentity into file to assign TSS to cells
    iD = handles;  % FISH-quant input data array is named handles
    
% Extract detection summary     
for j = 1:length(iD)
    for i = 1:length(iD(1,j).cell_summary)
        iD(j).cell_summary(i,1).dataset = j; 
    end
    for i = 1:length(iD(1,j).TS_summary)
        iD(j).TS_summary(1,i).dataset = j; 
    end
end
clear i j handles 

% Assign analyzed parameters from TSS detection to cell-summary 
cells_all = vertcat(iD(1,1:length(iD)).cell_summary); 
TSS_all = horzcat(iD(1,1:length(iD)).TS_summary);

for i = 1:length(cells_all)
    cells_all(i,1).TSS_count = length(cells_all(i,1).pos_TS); % number of TSS per cell defined by co-stain
end
clear i

% cytoplasmic RNA spots 
for i = 1:length(cells_all)     
    cells_all(i,1).N_cyt = ([cells_all(i,1).N_count]-[cells_all(i,1).N_nuc]);
end
clear i iD

% unique identifier for each cell and TSS based on strings of file-name and cell-name 
for i = 1:length(TSS_all)
    TSS_all(1,i).datasetSTR = strcat({'Set_'},num2str(TSS_all(1,i).dataset));
end
for i = 1:length(cells_all)
    cells_all(i,1).datasetSTR = strcat({'Set_'},num2str(cells_all(i,1).dataset));  
end
clear i

% unique ID TSS
for i = 1:length(TSS_all)
    temp1(i,1) = {TSS_all(1,i).file_name_list}';
    temp2(i,1) = {TSS_all(1,i).cell_label}';
    temp3(i,1) = {TSS_all(1,i).datasetSTR}';
    temp4 = strcat(temp3,temp2,temp1);
    TSS_Names_joined(i,1) = join(temp4{i,1});
end
clear i temp1 temp2 temp3 temp4

% unique ID cells
for i = 1:length(cells_all)
    temp1 = {cells_all(:,1).name_list}';
    temp2 = {cells_all(:,1).cell}';
    temp3 = {cells_all(:,1).datasetSTR}';  
    temp4 = strcat(temp3,temp2,temp1); 
    Cell_Names_joined(i,1) = join(temp4{i,1});
    cells_all(i,1).Cell_Names_joined = Cell_Names_joined(i,1);
end   
clear i temp1 temp2 temp3 temp4

% Assign Data from TSS_all into cells_all.TSS_info
for i = 1:length(cells_all)
    idx=find(strcmpi(cells_all(i).Cell_Names_joined,TSS_Names_joined));
    cells_all(i).TSS_info = TSS_all(:,idx);     
end
clear i idx TSS_Names_joined Cell_Names_joined
  
% Calculate mean intensity for all spots per cell 
for i = 1:length(cells_all)
    cells_all(i,1).mspotInt_raw = mean(cells_all(i,1).spots_detected(:,10));
end
clear i

X = [cells_all(:,1).N_count];
  
for i= 1 
    for j = 1:length(cells_all(i).spots_detected(:,10))       
        temp(i,j) = (cells_all(i,1).spots_detected(j,10));   
    end
    temp = temp'; 
end
clear i j

for i= 2:length(cells_all) 
    if cells_all(i).N_total ~= 0        
        for j = 1:length(cells_all(i).spots_detected(:,10)) 
            temp1(1,j) = (cells_all(i,1).spots_detected(j,10));   
        end
            if i == 2
                temp2 = vertcat(temp,temp1');
            else 
                temp2 = vertcat(temp2,temp1');
            end
    end
    clear temp1
end
spotInt_all = temp2; 
clear temp temp1 temp2 i j

% The following parameters are joined into TSS_information
    % TSS_count = number of identified TSS based on co-stain 
    % f = fraction of active promoters TSS_count/genomic loci
    % mIE_cell = mean spot intensity per cell (use mean of all spots in case of a low number of spots) 
    % IE_TSS = TSS intensity
    % M = RNAP2 occupancy
    % u = transcription rate 

for i = 1:length(cells_all)
% If no TSS was indentified set all parameters to '0
    if  cells_all(i,1).TSS_count==0 
            cells_all(i,1).TSS_info(1,1).mIEcell = cells_all(i,1).mspotInt_raw; 
            cells_all(i,1).TSS_info(1,1).IE_TSS=0; 
            cells_all(i,1).TSS_info(1,1).M=0; 
            cells_all(i,1).TSS_info(1,1).u=0; 
    end
end
clear i

for i = 1:length(cells_all)
% If one or more TSS were identified
    if cells_all(i,1).TSS_count ~=0
        for j = 1:length(cells_all(i).TSS_info)        
            cells_all(i,1).TSS_info(1,j).mIEcell = cells_all(i,1).mspotInt_raw;
            cells_all(i,1).TSS_info(1,j).IE_TSS = cells_all(i,1).TSS_info(1,j).TxSite_quant.TS_sum;            
            cells_all(i,1).TSS_info(1,j).M = cells_all(i,1).TSS_info(1,j).IE_TSS/(e*k*cells_all(i,1).TSS_info(1,j).mIEcell);
            cells_all(i,1).TSS_info(1,j).u = cells_all(i,1).TSS_info(1,j).M.*v./l;
        end
    end
end
for i = 1:length(cells_all)  
    if cells_all(i,1).TSS_count ~=0       
        for j = 1:length(cells_all(i).TSS_info)
            % if a TSS was identified based on co-stain but summed intensity is = 0, M is 1
            if cells_all(i,1).TSS_info(1,j).TxSite_quant.N_mRNA_integrated_int ~=0  % double check this-NOT USED I think
            if cells_all(i,1).TSS_info(1,j).TxSite_quant.TS_sum==0                 
               cells_all(i,1).TSS_info(1,j).IE_TSS = 0;
               cells_all(i,1).TSS_info(1,j).M = 1 ; 
               cells_all(i,1).TSS_info(1,j).u = cells_all(i,1).TSS_info(1,j).M.*v./l;
            else
                cells_all(i,1).TSS_info(1,j).M = cells_all(i,1).TSS_info(1,j).IE_TSS/(e*k*cells_all(i,1).TSS_info(1,j).mIEcell);
                cells_all(i,1).TSS_info(1,j).u = cells_all(i,1).TSS_info(1,j).M.*v./l;                                
                end                
            end
        end
     end
end
clear i j
%TSS parameters per cell
for i = 1:length(cells_all) 
        cells_all(i,1).M_sum = sum([cells_all(i,1).TSS_info(1,:).M]); 
        cells_all(i,1).u_sum = sum([cells_all(i,1).TSS_info(1,:).u]);
        cells_all(i,1).f = cells_all(i,1).TSS_count/n;
        cells_all(i,1).d = (cells_all(i,1).u_sum.*n*cells_all(i,1).f)./(X(i));
end
clear i j

% Parameters for each TSS to compare individual transcription sites
for i = 1:length(cells_all)
      for j = 1:length(cells_all(i).TSS_info)
           c1_ind_IE(i,j) =  cells_all(i,1).TSS_info(1,j).IE_TSS; 
           c1_ind_M(i,j) =  cells_all(i,1).TSS_info(1,j).M;
           c1_ind_u(i,j) = cells_all(i,1).TSS_info(1,j).u; 
      end
end    
clear i j

M_cell = [cells_all(:,1).M_sum]'; 
u_cell = [cells_all(:,1).u_sum]';
d = [cells_all(:,1).d]';
f = [cells_all(:,1).f]';  
    
 data1 = c1_ind_IE(:,1); 
 data2 = c1_ind_IE(:,2);
 data3 = c1_ind_IE(:,3);
 temp1 = data1(data1~=0);
 temp2 = data2(data2~=0); 
 temp3 = data3(data3~=0); 
 Ind_IETSS = vertcat (temp1,temp2,temp3);  
 clear data1 data2 data3 temp1 temp2 temp3 clear c1_ind_IE
 
 data1 = c1_ind_u (:,1);
 data2 = c1_ind_u (:,2);
 data3 = c1_ind_u (:,3);
 temp1 = data1(data1~=0);
 temp2 = data2(data2~=0); 
 temp3 = data3(data3~=0); 
 Ind_u = vertcat (temp1,temp2,temp3);
 clear data1 data2 data3  temp1 temp2 temp3 c1_ind_u
 
 data1 = c1_ind_M (:,1);
 data2 = c1_ind_M (:,2);
 data3 = c1_ind_M (:,3);
 temp1 = data1(data1~=0);
 temp2 = data2(data2~=0); 
 temp3 = data3(data3~=0); 
 Ind_M = vertcat (temp1,temp2,temp3);
 clear data1 data2 data3 temp1 temp2 temp3 c1_ind_M 
